import * as tempApi from "@/api/temp";
import * as chartTempApi from "@/api/chartTemp";
import {deepCopy} from "@/utils";

const useTempStore = defineStore('temp', () => {
  const state = reactive({
    visualCategoryList: [],
    visualCategoryTree: [],
    visualCategoryParents: [],
    chartCategoryList: [],
    chartCategoryTree: [],
    chartCategoryParents: [],
  })
  const getVisualCategory = () => {
    return new Promise((resolve, reject) => {
      tempApi.categoryList().then(res => {
        // 设置平铺列表
        state.visualCategoryList = res.data || []
        // 设置分类树
        let parents = state.visualCategoryList.filter(i => !i.parentId).sort((b, a) => b.orderIndex - a.orderIndex)
        parents.forEach(p => {
          let children = state.visualCategoryList.filter(i => i.parentId === p.id)
          if (children) {
            p.children = children.sort((b, a) => b.orderIndex - a.orderIndex)
            children.forEach(pp => {
              let children2 = state.visualCategoryList.filter(i => i.parentId === pp.id)
              if (children2) pp.children = children2.sort((b, a) => b.orderIndex - a.orderIndex)
            })
          }
        })
        state.visualCategoryTree = parents
        // 设置父级分类树
        let categoryList = deepCopy(state.visualCategoryList)
        categoryList.forEach(i => delete i.children)
        let parents2 = categoryList.filter(i => !i.parentId).sort((b, a) => b.orderIndex - a.orderIndex)
        parents2.forEach(p => {
          let children = categoryList.filter(i => i.parentId === p.id)
          if (children) p.children = children.sort((b, a) => b.orderIndex - a.orderIndex)
        })
        parents2.unshift({name: '顶级分类', id: 0})
        state.visualCategoryParents = parents2
        resolve()
      })
    })
  }
  const getVisualCategorySearched = key => {
    let categoryTree2 = deepCopy(state.visualCategoryTree)
    let res = [
      {name: '全部分类', id: 0, parentId: 0},
    ]
    if (!key) {
      return [...res, ...categoryTree2]
    } else {
      let visualCategoryList = deepCopy(state.visualCategoryList)
      let searched = visualCategoryList.filter(i => i.name.includes(key))
      searched.forEach(i => delete i.children)
      return [...res, ...searched]
    }
  }
  const getChartCategory = () => {
    return new Promise((resolve, reject) => {
      chartTempApi.categoryList().then(res => {
        // 设置平铺列表
        state.chartCategoryList = res.data || []
        // 设置分类树
        let parents = state.chartCategoryList.filter(i => !i.parentId).sort((b, a) => b.orderIndex - a.orderIndex)
        parents.forEach(p => {
          let children = state.chartCategoryList.filter(i => i.parentId === p.id)
          if (children) {
            p.children = children.sort((b, a) => b.orderIndex - a.orderIndex)
            children.forEach(pp => {
              let children2 = state.chartCategoryList.filter(i => i.parentId === pp.id)
              if (children2) pp.children = children2.sort((b, a) => b.orderIndex - a.orderIndex)
            })
          }
        })
        state.chartCategoryTree = parents
        // 设置父级分类树
        let categoryList = deepCopy(state.chartCategoryList)
        categoryList.forEach(i => delete i.children)
        let parents2 = categoryList.filter(i => !i.parentId).sort((b, a) => b.orderIndex - a.orderIndex)
        parents2.forEach(p => {
          let children = categoryList.filter(i => i.parentId === p.id)
          if (children) p.children = children.sort((b, a) => b.orderIndex - a.orderIndex)
        })
        parents2.unshift({name: '顶级分类', id: 0})
        state.chartCategoryParents = parents2
        resolve()
      })
    })
  }
  const getChartCategorySearched = key => {
    let categoryTree2 = deepCopy(state.chartCategoryTree)
    let res = [
      {name: '全部分类', id: 0, parentId: 0},
    ]
    if (!key) {
      return [...res, ...categoryTree2]
    } else {
      let chartCategoryList = deepCopy(state.chartCategoryList)
      let searched = chartCategoryList.filter(i => i.name.includes(key))
      searched.forEach(i => delete i.children)
      return [...res, ...searched]
    }
  }
  return {
    state,
    getVisualCategory,
    getChartCategory,
    getVisualCategorySearched,
    getChartCategorySearched,
  }
})

export default useTempStore