export const exportExcel = (XLSX, filename, data) => {
  let exc = XLSX.utils.book_new(); // 初始化一个excel文件
  let exc_data = XLSX.utils.aoa_to_sheet(data);   // 传入数据 , 到excel
  // 将文档插入文件并定义名称
  XLSX.utils.book_append_sheet(exc, exc_data, filename);
  // 执行下载
  XLSX.writeFile(exc, filename + '.xlsx');
}

/**
 * 递归深拷贝函数
 * @param {任何类型} obj - 需要拷贝的对象
 * @param {WeakMap} map - 用于存储已拷贝的对象与原对象之间的映射关系，默认为空弱Map
 * @returns {任何类型} - 拷贝后的对象
 */
export const deepCopy = (obj, map = new WeakMap()) => {
  // 如果 obj 是 Date 类型，则创建一个新的 Date 对象并返回
  if (obj instanceof Date) return new Date(obj)
  // 如果 obj 是 RegExp 类型，则创建一个新的 RegExp 对象并返回
  if (obj instanceof RegExp) return new RegExp(obj)

  // 如果 obj 为 null 或者不是对象，则直接返回
  if (obj === null || typeof obj !== 'object') {
    return obj
  }
  // 如果 map 中已存在 obj 的映射关系，则直接返回该映射关系的值
  if (map.has(obj)) {
    return map.get(obj)
  } else {
    // 创建一个新的 obj 的构造函数的实例
    const res = new obj.constructor()
    // 将 res 添加到 map 中作为 obj 的映射关系
    map.set(obj, res)
    if (res.set) { // Set Map WeakMap WeakSet
      if (res.forEach) { // Set Map
        // 如果 res 的原型链上存在 set 方法和 forEach 方法
        // 使用 forEach 方法遍历 obj 的每个元素，并将其拷贝后设置到 res 中
        obj.forEach((value, key) => {
          res.set(key, deepCopy(value, map))
        })
      } else { // WeakMap WeakSet
        // 如果 res 的原型链上不存在 forEach 方法
        // 抛出异常，提示不支持拷贝 obj 的类型
        throw new Error(obj.constructor.name + '不支持拷贝')
      }
    } else {
      // 如果 res 的原型链上不存在 set 方法
      // 使用 for...in 循环遍历 obj 的每个属性，并将其拷贝后赋值到 res
      for (let key in obj) {
        if (obj.hasOwnProperty(key)) {
          res[key] = deepCopy(obj[key], map)
        }
      }
    }
    return res
  }
}

export function debounce(func, wait) {
  let timeout;
  return function () {
    let context = this; // 保存this指向
    let args = arguments; // 拿到event对象
    if (timeout) clearTimeout(timeout)
    timeout = setTimeout(function () {
      func.apply(context, args)
    }, wait);
  }
}
export function copyContent(content) {
  var copyDom = document.createElement('div');
  copyDom.innerText = content;
  copyDom.style.position = 'absolute';
  copyDom.style.top = '0px';
  copyDom.style.right = '-9999px';
  document.body.appendChild(copyDom);
  //创建选中范围
  var range = document.createRange();
  range.selectNode(copyDom);
  //移除剪切板中内容
  window.getSelection().removeAllRanges();
  //添加新的内容到剪切板
  window.getSelection().addRange(range);
  //复制
  var successful = document.execCommand('copy');
  copyDom.parentNode.removeChild(copyDom);
  try {
    var msg = successful ? "successful" : "failed";
    console.log('Copy command was : ' + msg);
  } catch (err) {
    console.log('Oops , unable to copy!');
  }
}
let totalChart = 'abcdefghijklmiopqrstuvwxyzABCDEFGHIJKLMIOPQRSTUVWXYZ';
export function getRandomStr(length) {
  return new Array(length)
    .fill('')
    .map(i => totalChart[parseInt(Math.random() * totalChart.length)])
    .join('');
}
export function objectToParam(params) {
  const paramsUrl = new URLSearchParams('')
  for (let key in params) {
    if (params.hasOwnProperty(key)) {
      // const value = encodeURIComponent(params[key]) // 编码特殊字符
      const value = params[key]
      paramsUrl.append(key, value) // 添加键值对到 URLSearchParams 对象
    }
  }
  return paramsUrl.toString()
}